@@ -4,17 +4,18 @@ and block parameter/query input.
 """
 import json
 import os
-from typing import Callable
+from typing import Callable, Union, List
 from contextlib import ContextDecorator
-import pathlib
+from pathlib import Path
 from enum import Enum
 import shutil
 import base64
+from datetime import datetime, timedelta
 
 import rasterio as rio
 from rasterio import warp
 from shapely.geometry import box, mapping
-
+from dateutil.parser import parse
 from geojson import FeatureCollection, Feature
 
 from .stac import STACQuery
@@ -27,42 +28,41 @@ def ensure_data_directories_exist():
     """Creates required directories for any block input and output (`/tmp/input`,
     `tmp/output`, `/tmp/quicklooks`).
     """
-    pathlib.Path("/tmp/input/").mkdir(parents=True, exist_ok=True)
-    pathlib.Path("/tmp/output/").mkdir(parents=True, exist_ok=True)
-    pathlib.Path("/tmp/quicklooks/").mkdir(parents=True, exist_ok=True)
+    Path("/tmp/input/").mkdir(parents=True, exist_ok=True)
+    Path("/tmp/output/").mkdir(parents=True, exist_ok=True)
+    Path("/tmp/quicklooks/").mkdir(parents=True, exist_ok=True)
 
 
-def setup_test_directories(test_dir: pathlib.Path):
-    """Creates given test directory and empty input/output/quicklook subfolders.
-
-    Arguments:
-        test_dir: A directory to store temporary files (usually `/tmp` or
-            `/tmp/e2e_test`)
+def setup_test_directories(
+    test_dir: Union[str, Path], clean_subdirs: bool = True
+) -> List[Path]:
+    """Creates given test directory and empty subdirs "input", "output", "quicklooks".
 
+    Args:
+        test_dir: A directory to store temporary files (usually `/tmp` or `/tmp/e2e_test`)
+        clean_subdirs: Remove all files in the subdirs "input", "output", "quicklooks".
+    Returns:
+        List of sub directories Paths depending onthe selection in sub_dirs.
     """
+    test_dir = Path(test_dir)
     test_dir.mkdir(parents=True, exist_ok=True)
 
-    cleanup_test_directories(test_dir)
-
-
-def cleanup_test_directories(test_dir: pathlib.Path):
-    """Deletes given test directory.
-
-    Arguments:
-        test_dir: A directory to store temporary files (usually `/tmp` or \
-            `/tmp/e2e_test`)
-    """
-    if test_dir.exists():
-        for folder in ["input", "output", "quicklooks"]:
+    subdirs = ["input", "output", "quicklooks"]
+    subdir_paths = [test_dir / subdir for subdir in subdirs]
+    for subdir_path in subdir_paths:
+        if clean_subdirs:
+            # Should not remove other files/dirs in test_dir, since often /tmp.
+            subdir_path.mkdir(parents=True, exist_ok=True)
             try:
-                shutil.rmtree(test_dir / folder)
-                pathlib.Path(test_dir / folder).mkdir(parents=True, exist_ok=True)
+                shutil.rmtree(subdir_path)
             # Deleting subfolder sometimes does not work in temp, then remove all subfiles.
             except (PermissionError, OSError):
-                pathlib.Path(test_dir / folder).mkdir(parents=True, exist_ok=True)
-                files_to_delete = pathlib.Path(test_dir / folder).rglob("*.*")
+                files_to_delete = subdir_path.rglob("*.*")
                 for file_path in files_to_delete:
                     file_path.unlink()
+        subdir_path.mkdir(parents=True, exist_ok=True)
+
+    return subdir_paths
 
 
 class TestDirectoryContext(ContextDecorator):
@@ -70,12 +70,12 @@ class TestDirectoryContext(ContextDecorator):
     context is closed.
     """
 
-    def __init__(self, test_dir: pathlib.Path):
+    def __init__(self, test_dir: Path):
         """
         Example:
             ```python
-            with TestDirectoryContext(pathlib.Path("/tmp")) as test_dir:
-                block.process(dir=pathlib.Path("/tmp"))
+            with TestDirectoryContext(Path("/tmp")) as test_dir:
+                block.process(dir=Path("/tmp"))
             ```
 
         Arguments:
@@ -84,7 +84,7 @@ class TestDirectoryContext(ContextDecorator):
         """
         self.test_dir = test_dir
 
-    def __enter__(self) -> pathlib.Path:
+    def __enter__(self) -> Path:
         """Context entry point.
 
         Returns:
@@ -94,8 +94,8 @@ class TestDirectoryContext(ContextDecorator):
         return self.test_dir
 
     def __exit__(self, *exc):
-        """Context exit point. Deletes temporary directory."""
-        cleanup_test_directories(self.test_dir)
+        """Context exit point. Cleans up test subdirs."""
+        setup_test_directories(test_dir=self.test_dir)
         return False
 
 
@@ -264,3 +264,52 @@ def decode_str_base64(string: str) -> str:
     str_encoded_byte = bytes(string, "utf-8")
     str_decoded_byte = base64.b64decode(str_encoded_byte)
     return str_decoded_byte.decode("utf-8")
+
+
+def get_timeperiod(
+    duration: int = 365,
+    start: int = -365,
+    start_date: str = None,
+    end_date: str = None,
+) -> str:
+    """
+    Generates time period string from any combinations of duration(days) & start(days),
+    duration(days) & start_date or start_date & end_date parameters. Defaults to 1 year till today.
+
+    Most relevant for selecting timeperiods for testing of rolling archives.
+
+    Args:
+        duration: Time period duration in days.
+        start: Time period start from today in days.
+        start_date: Time period start date, format string YYYY-MM-DD.
+        end_date: Time period end date, format string YYYY-MM-DD.
+
+    Returns:
+        time period string with variable time and fixed clock time, in the format
+        "2019-07-01T00:00:00+00:00/2029-07-10T23:59:59+00:00"
+    """
+    if start_date is not None and end_date is not None:
+        start_date = parse(start_date).replace(
+            hour=0, minute=0, second=0, microsecond=0
+        )
+        end_date = parse(end_date).replace(hour=0, minute=0, second=0, microsecond=0)
+    elif start_date is not None and duration is not None:
+        start_date = parse(start_date).replace(
+            hour=0, minute=0, second=0, microsecond=0
+        )
+        end_date = start_date + timedelta(days=+duration)
+    elif duration is not None and start is not None:
+        today = datetime.today().replace(hour=0, minute=0, second=0, microsecond=0)
+        start_date = today + timedelta(days=+start)
+        end_date = start_date + timedelta(days=+duration)
+    else:
+        raise ValueError(
+            "Only the combinations `1. duration(months) & start(months), "
+            "2. duration(months) & start_date or 3. start_date & end_date` are allowed."
+        )
+
+    start_date = start_date.strftime("%Y-%m-%d")
+    end_date = end_date.strftime("%Y-%m-%d")
+
+    time = f"{start_date}T00:00:00+00:00/{end_date}T23:59:59+00:00"
+    return time
