@@ -3,10 +3,11 @@ Utilities for handling [SpatioTemporal Asset Catalog (STAC)](https://stacspec.or
 """
 
 from __future__ import annotations
+from dataclasses import dataclass, field
 
 import json
 from decimal import Decimal
-from typing import Union, Tuple, Iterable, cast, Optional, Callable, Any
+from typing import Union, Tuple, List, Iterable, Optional, Callable, Any
 
 import shapely.geometry
 
@@ -23,44 +24,76 @@ Coordinates = Iterable[Number]
 BoundingBox = Tuple[Number, Number, Number, Number]
 
 
+@dataclass
 class STACQuery:
     """
-    Object representing a STAC query as used by data blocks.
+    Object representing a STAC query as used by data blocks. Standardized
+    arguments are:
+
+    Arguments:
+        ids: Unique identifiers to search for.
+        bbox: A bounding box type geometry.
+        intersects: A geometry to use for searching with intersection operator.
+        contains: A geometry to use for searching with contains operator.
+        time: Time string in RFC3339 (for points in time) or RFC3339/RFC3339
+            (for datetime ranges).
+        limit: Maximum number of returned elements.
+        time_series: List of time strings in RFC3339 (for points in time) or RFC3339/RFC3339
+            (for datetime ranges).
+
+    Additional arguments can be passed to the constructor.
     """
 
-    def __repr__(self):
-        return (
-            f"STACQuery(ids={self.ids}, bbox={self.bbox}, intersects={self.intersects}, "
-            f"contains={self.contains}, time={self.time}, time_series={self.time_series}, "
-            f"limit={self.limit})"
-        )
+    ids: Optional[List[str]] = field(default=None)
+    bbox: Optional[BoundingBox] = field(default=None)
+    intersects: Optional[Geometry] = field(default=None)
+    contains: Optional[Geometry] = field(default=None)
+    time: Optional[str] = field(default=None)
+    time_series: Optional[List[str]] = field(default=None)
+    limit: Optional[int] = field(default=1)
 
-    def __init__(
-        self,
-        ids: Optional[list] = None,
-        bbox: Optional[BoundingBox] = None,
-        intersects: Optional[Geometry] = None,
-        contains: Optional[Geometry] = None,
-        time: Optional[str] = None,
-        limit: Optional[int] = None,
-        time_series: Optional[list] = None,
-        **kwargs,
-    ):  # pylint: disable=too-many-arguments
+    def __new__(cls, **kwargs):
         """
-        Arguments:
-            ids: Unique identifiers to search for.
-            bbox: A bounding box type geometry.
-            intersects: A geometry to use for searching with intersection operator.
-            contains: A geometry to use for searching with contains operator.
-            time: Time string in RFC3339 (for points in time) or RFC3339/RFC3339
-                (for datetime ranges).
-            limit: Maximum number of returned elements.
-            time_series: List of time strings in RFC3339 (for points in time) or RFC3339/RFC3339
-                (for datetime ranges).
-            kwargs: Arbitrary query arguments.
+        Inspired by https://stackoverflow.com/a/63291704/14236301
+        Adds attributes to the data class if they are passed in
+        input dict in __init__, even if they are not explicitly defined as
+        field attributes.
         """
+        try:
+            initializer = cls.__initializer
+        except AttributeError:
+            # Store the original init on the class in a different place
+            cls.__initializer = initializer = cls.__init__
+            # replace init with something harmless
+            cls.__init__ = lambda *a, **k: None
 
-        if bbox and intersects or intersects and contains or contains and bbox:
+        # code from adapted from Arne
+        added_args = {}
+        for name in list(kwargs.keys()):
+            if name not in cls.__annotations__:
+                added_args[name] = kwargs.pop(name)
+
+        ret = object.__new__(cls)
+        initializer(ret, **kwargs)
+        # ... and add the new ones by hand
+        for new_name, new_val in added_args.items():
+            setattr(ret, new_name, new_val)
+
+        return ret
+
+    def __post_init__(self):
+        """
+        Post processes the passed arguments to make sure they are valid,
+        i.e. only one of "bbox", "intersects" or "contains" is passed.
+        """
+        if (
+            self.bbox
+            and self.intersects
+            or self.intersects
+            and self.contains
+            or self.contains
+            and self.bbox
+        ):
             raise UP42Error(
                 SupportedErrors.WRONG_INPUT_ERROR,
                 """Only one of the following query parameters is
@@ -70,15 +103,17 @@ class STACQuery:
                 * contains.""",
             )
 
-        self.__dict__.update(
-            kwargs
-        )  # At the front so we do not accidentally overwrite some members
-        self.ids = ids
-        self.bbox = bbox
-        self.intersects = intersects
-        self.contains = contains
+        if self.time and self.time_series:
+            raise UP42Error(
+                SupportedErrors.WRONG_INPUT_ERROR,
+                """Only one of the following query parameters is
+                allowed at a query at any given time:
+                * time
+                * time_series
+                """,
+            )
 
-        if not STACQuery.validate_datetime_str(time):
+        if not self.validate_datetime_str(self.time):
             raise UP42Error(
                 SupportedErrors.WRONG_INPUT_ERROR,
                 """Time string could not be validated.
@@ -86,18 +121,16 @@ class STACQuery:
                 <RFC3339> (for points in time)
                 <RFC3339>/<RFC3339> (for datetime ranges)""",
             )
-        self.time = time
 
-        self.limit = 1 if limit is None else limit
-        if limit is not None and limit < 0:
+        if self.limit is not None and self.limit < 1:
             logger.warning(
-                "WARNING: limit parameter cannot be < 0, and has been automatically set to 1"
+                "WARNING: limit parameter cannot be < 1, and has been automatically set to 1"
             )
             self.limit = 1
 
-        if time_series is not None:
-            for datestr in time_series:
-                if not STACQuery.validate_datetime_str(datestr):
+        if self.time_series is not None:
+            for datestr in self.time_series:
+                if not self.validate_datetime_str(datestr):
                     raise UP42Error(
                         SupportedErrors.WRONG_INPUT_ERROR,
                         """Time string from time_series could not be validated.
@@ -105,7 +138,6 @@ class STACQuery:
                         <RFC3339> (for points in time)
                         <RFC3339>/<RFC3339> (for datetime ranges)""",
                     )
-        self.time_series = time_series
 
     def bounds(self) -> BoundingBox:
         """
@@ -153,6 +185,17 @@ class STACQuery:
                 * contains.""",
             )
 
+    def get_param_if_exists(self, key: str, default_value: Any = None) -> Any:
+        """
+        Get a query parameter if exists in he STAC query object. Returns
+        default_value if not in the query.
+
+        Arguments:
+            key: An identifier of the parameter
+            default_value: The default return value if the parameter does not exist.
+        """
+        return getattr(self, key, default_value)
+
     def set_param_if_not_exists(self, key: str, value: Any):
         """
         Set a query parameter if it does not exist in the STAC query object.
@@ -161,8 +204,8 @@ class STACQuery:
             key: An identifier of the parameter
             value: The parameter value to set.
         """
-        if not key in self.__dict__:
-            self.__dict__[key] = value
+        if self.get_param_if_exists(key) is None:
+            setattr(self, key, value)
 
     @classmethod
     def from_json(cls, json_data: str) -> STACQuery:
@@ -199,65 +242,7 @@ class STACQuery:
                 to the official block documentation or block specification.""",
             )
 
-        bbox: Optional[BoundingBox] = cast(BoundingBox, dict_data.get("bbox")) if not (
-            dict_data.get("bbox") is None
-            or dict_data.get("bbox") == ""
-            or dict_data.get("bbox") == {}
-        ) else None
-
-        intersects: Optional[Geometry] = Geometry(
-            **dict_data.get("intersects")
-        ) if not (
-            dict_data.get("intersects") is None
-            or dict_data.get("intersects") == ""
-            or dict_data.get("intersects") == {}
-        ) else None
-
-        contains: Optional[Geometry] = Geometry(**dict_data.get("contains")) if not (
-            dict_data.get("contains") is None
-            or dict_data.get("contains") == ""
-            or dict_data.get("contains") == {}
-        ) else None
-
-        time: Optional[str] = dict_data.get("time") if not dict_data.get(
-            "time"
-        ) == "" else None
-        limit: Optional[int] = dict_data.get("limit") if not dict_data.get(
-            "limit"
-        ) == "" else None
-        ids: Optional[list] = dict_data.get("ids") if not dict_data.get(
-            "ids"
-        ) == "" else None
-        time_series: Optional[list] = dict_data.get("time_series") if not dict_data.get(
-            "time_series"
-        ) == "" else None
-
-        known_filters = [
-            "bbox",
-            "intersects",
-            "contains",
-            "time",
-            "limit",
-            "geometry",
-            "bounds",
-            "ids",
-            "time_series",
-        ]
-
-        truncated_dict_data = {
-            key: dict_data[key] for key in dict_data if key not in known_filters
-        }
-
-        return STACQuery(
-            ids=ids,
-            bbox=bbox,
-            intersects=intersects,
-            contains=contains,
-            time=time,
-            limit=limit,
-            time_series=time_series,
-            **truncated_dict_data,
-        )
+        return STACQuery(**dict_data)
 
     @staticmethod
     def validate_datetime_str(string: Optional[str]) -> bool:
